package com.tocea.easycoverage.framework;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.api.IEasycoverageChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;

/**
 * 
 * A checker that is applied on a class.
 * 
 * @author Romain Delamare
 * 
 */
public abstract class ClassChecker implements IEasycoverageChecker {

	final protected IInstanceProvider	provider;
	final protected Class<?>			clazz;

	public ClassChecker(final IInstanceProvider provider, final Class<?> clazz) {
		this.provider = provider;
		this.clazz = clazz;
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + "-"
				+ (clazz == null ? "" : clazz.getName());
	}

	public final boolean shouldCheck() {
		return clazz != null
				&& !clazz.isAnnotationPresent(SkipAutomaticTests.class)
				&& shouldCheckClass();
	}

	protected abstract boolean shouldCheckClass();

}
