package com.tocea.easycoverage.framework;

import java.lang.reflect.Method;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.api.IEasycoverageChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * A checker that is applied on a method.
 * 
 * @author Romain Delamare
 * 
 */
public abstract class MethodChecker implements IEasycoverageChecker {

	final protected IInstanceProvider	provider;
	final protected Method				method;

	public MethodChecker(final IInstanceProvider provider, final Method method) {
		this.provider = provider;
		this.method = method;
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + "-"
				+ Utils.getQualifiedName(method);
	}

	public final boolean shouldCheck() {
		return method != null
				&& !method.isAnnotationPresent(SkipAutomaticTests.class)
				&& !method.getDeclaringClass().isAnnotationPresent(
						SkipAutomaticTests.class) && shouldCheckMethod();
	}

	protected abstract boolean shouldCheckMethod();

}
