package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that verifies that no array index out of bound are thrown. It will
 * try to invoke each method with min and max values.
 * 
 * @author Romain Delamare
 * 
 */
public class ArrayIndexOutOfBoundExceptionChecker extends MethodChecker {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(ArrayIndexOutOfBoundExceptionChecker.class);

	public ArrayIndexOutOfBoundExceptionChecker(
			final IInstanceProvider provider, final Method method) {
		super(provider, method);
	}

	public void check() {
		if (method == null)
			return;
		final Object[] maxParams = Utils.generateMaxValues(method
				.getParameterTypes());
		final Object[] minParams = Utils.generateMinValues(method
				.getParameterTypes());
		final Object instance = provider
				.getInstance(method.getDeclaringClass());

		try {
			method.invoke(instance, maxParams);
			method.invoke(instance, minParams);
		} catch (final IllegalArgumentException e) {
			LOGGER.error("Error during invocation of {}: illegal arguments.",
					method);
		} catch (final IllegalAccessException e) {
			LOGGER.error("Error during invocation of {}: illegal access.",
					method);
		} catch (final InvocationTargetException e) {
			if (e.getCause() instanceof ArrayIndexOutOfBoundsException)
				throw new EasycoverageAssertionError(
						"ArrayIndexOutOfBoundException thrown.", e.getCause());
		}
	}

	@Override
	public boolean shouldCheckMethod() {
		return method != null && method.getParameterTypes().length > 0;
	}

}
