package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;
import com.tocea.easycoverage.framework.utils.Utils;

/**
 * 
 * Checkers that verifies, for a getter/setter pair, that the value return by
 * the getter is the value previously set.
 * 
 * @author Romain Delamare
 * 
 */
public class SetterChecker extends MethodChecker {

	private static final Logger	LOGGER	= LoggerFactory
												.getLogger(SetterChecker.class);

	public SetterChecker(final IInstanceProvider provider, final Method method) {
		super(provider, method);
	}

	public void check() {
		if (provider == null || method == null)
			return;
		final Object o = provider.getInstance(method.getDeclaringClass());
		final Object parameter = provider.getInstance(method
				.getParameterTypes()[0]);
		final Method getter = Utils.getGetterForSetter(method);
		try {
			method.invoke(o, parameter);
			final Object result = getter.invoke(o);
			if (!method.getParameterTypes()[0].isPrimitive()
					&& result != parameter
					|| method.getParameterTypes()[0].isPrimitive()
					&& result != null && !result.equals(parameter))
				throw new EasycoverageAssertionError(
						"Accessor error: set value was: " + parameter
								+ " but retrieved value is: " + result);
		} catch (final IllegalArgumentException e) {
			LOGGER.error("Error during invocation of {}: illegal argument.",
					method);
		} catch (final IllegalAccessException e) {
			LOGGER.error("Error during invocation of {}: illegal access.",
					method);
		} catch (final InvocationTargetException e) {
			throw new RuntimeException(e.getCause());
		}
	}

	@Override
	public boolean shouldCheckMethod() {
		return Utils.isSetter(method)
				&& Utils.getGetterForSetter(method) != null && provider != null
				&& provider.canProvide(method.getParameterTypes()[0]);
	}

}
