package com.tocea.easycoverage.framework.checkers;

import java.lang.reflect.Method;

import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.exception.EasycoverageAssertionError;

/**
 * 
 * Checkers that vefiries that toString does not return null, for each class
 * that implements it.
 * 
 * @author Romain Delamare
 * 
 */
public class ToStringNotNullChecker extends ClassChecker {

	public ToStringNotNullChecker(final IInstanceProvider provider,
			final Class<?> clazz) {
		super(provider, clazz);
	}

	public void check() {
		if (provider != null) {
			final Object o = provider.getInstance(clazz);
			if (o != null && o.toString() == null)
				throw new EasycoverageAssertionError(
						"toString should not return null");
		}
	}

	@Override
	public boolean shouldCheckClass() {
		Method toString = null;
		if (clazz != null)
			try {
				toString = clazz.getDeclaredMethod("toString");
			} catch (final SecurityException e) {
				// nothing to do
			} catch (final NoSuchMethodException e) {
				// nothing to do
			}
		return toString != null;
	}

}
